//-------------------------------------------------------------------
// Media.h
//-------------------------------------------------------------------
// ACD SYSTEMS INTERNATIONAL INC.
// Copyright © 1994 - 2020 ACD Systems International Inc.
// All Rights Reserved
//
// NOTICE: ACD Systems International Inc. ("ACD") permits you to use,
// modify, and distribute this file in accordance with the terms of
// the ACDSee SDK license agreement accompanying it. If you have 
// received this file from a source other than ACD, then your use,
// modification, or distribution of it requires the prior written 
// permission of ACD.
//-------------------------------------------------------------------

#pragma once

#include <windows.h>

/////////////
// MD_Item //
/////////////
struct Media_Item
{
   Media_Item() : nDataID(0), nTypeID(0), nCount(0), dwData(0), pNext(NULL) {};

   Media_Item( const WORD _nDataID, const WORD _nTypeID, const UINT _nCount, Media_Item* _pNext)
   {
      nDataID = _nDataID;
      nTypeID = _nTypeID;
      nCount  = _nCount;
      pNext = _pNext; 
   };

   WORD   nDataID;     // Data item ID
   WORD   nTypeID;     // Data item type
   UINT   nCount;      // Number of values of the indicated type
   union
   {
      void*  pData;    // Data as void pointer
      char*  pszData;  // Data as string
      DWORD  dwData;   // Data as a 32-bit value
      BYTE   aData[4]; // Data as an array of four bytes
      SHORT  sData[2]; // Data as an array of two shorts
   };
   Media_Item* pNext;
};

//////////////////////
// Data type values //
//////////////////////
enum
{
   MEDIA_TYPE_BYTE      =  1, // 8-bit  unsigned integer
   MEDIA_TYPE_STRING    =  2, // 8-bit byte that contains a NULL-terminated string
   MEDIA_TYPE_SHORT     =  3, // 16-bit unsigned integer
   MEDIA_TYPE_LONG      =  4, // 32-bit unsigned integer
   MEDIA_TYPE_RATIONAL  =  5, // Two 32-bit unsigned integers; the first represent the numerator of a fraction; the second, the denominator
   MEDIA_TYPE_SBYTE     =  6, // 8-bit  signed integer
   MEDIA_TYPE_UNDEFINED =  7, // 8-bit byte that may contain anything, depending on the definition of the field
   MEDIA_TYPE_SSHORT    =  8, // 16-bit signed integer
   MEDIA_TYPE_SLONG     =  9, // 32-bit signed integer
   MEDIA_TYPE_SRATIONAL = 10, // Two 32-bit signed integers; the first represent the numerator of a fraction; the second, the denominator
   MEDIA_TYPE_FLOAT     = 11, // Single precision (32-bit) IEEE format floating point
   MEDIA_TYPE_DOUBLE    = 12, // Double precision (64-bit) IEEE format floating point
};

static const int Media_NumTypes = 12;
static const int Media_TypeSizeTable[Media_NumTypes+1] = { 0, 1, 1, 2, 4, 8, 1, 1, 2, 4, 8, 4, 8 };


#define MEDIA_TYPE_UNKNOWN			   0
#define MEDIA_TYPE_IMAGE			   1
#define MEDIA_TYPE_VIDEO			   2
#define MEDIA_TYPE_AUDIO			   4

#define MEDIA_MEDIATYPE 			   0xf000   // MEDIA_TYPE_LONG: media type
#define MEDIA_LENGTH				      0xf001   // MEDIA_TYPE_DOUBLE: length in secs
#define MEDIA_FRAMERATE				   0xf002   // MEDIA_TYPE_DOUBLE: fps
#define MEDIA_CHANNEL				   0xf003   // MEDIA_TYPE_LONG: Mono = 1, Stereo = 2
#define MEDIA_SAMPLEPERSEC			   0xf004   // MEDIA_TYPE_LONG: common value --> 8,000, 11,025, 22,050 and 44,100
#define MEDIA_BITSPERSAMPLE			0xf005   // MEDIA_TYPE_LONG: 8, 16 or 0(if the audio is compressed) 


// MP3 ID3 Tags

enum // MEDIA_MP3ID3_MPEGVERSION
{
	MEDIA_MPEGVER_NA,			// reserved, N/A
	MEDIA_MPEGVER_25,			// 2.5
	MEDIA_MPEGVER_2,			   // 2.0 (ISO/IEC 13818-3)
	MEDIA_MPEGVER_1			   // 1.0 (ISO/IEC 11172-3)
};

enum // MEDIA_MP3ID3_CHANNELMODE
{
	MEDIA_CM_STEREO,
	MEDIA_CM_JOINT_STEREO,
	MEDIA_CM_DUAL_CHANNEL,
	MEDIA_CM_SINGLE_CHANNEL
};

enum // MEDIA_MP3ID3_EMPHASIS
{
	MEDIA_EM_NONE,
	MEDIA_EM_50_15_MS,		   // 50/15 ms
	MEDIA_EM_RESERVED,
	MEDIA_EM_CCIT_J17			   // CCIT J.17
};

#define MEDIA_MP3ID3_ARTIST			0xf100	// MEDIA_TYPE_STRING
#define MEDIA_MP3ID3_TITLE			   0xf101	// MEDIA_TYPE_STRING
#define MEDIA_MP3ID3_ALBUM			   0xf102	// MEDIA_TYPE_STRING
#define MEDIA_MP3ID3_COMMENT		   0xf103	// MEDIA_TYPE_STRING
#define MEDIA_MP3ID3_YEAR			   0xf104	// MEDIA_TYPE_STRING
#define MEDIA_MP3ID3_TRACK			   0xf105  // MEDIA_TYPE_LONG
#define MEDIA_MP3ID3_GENREINDEX		0xf106  // MEDIA_TYPE_LONG
#define MEDIA_MP3ID3_GENRENAME		0xf107  // MEDIA_TYPE_STRING
#define MEDIA_MP3ID3_LENGTH			0xf108  // MEDIA_TYPE_LONG
#define MEDIA_MP3ID3_MPEGVERSION	   0xf109  // MEDIA_TYPE_LONG
#define MEDIA_MP3ID3_MPEGLAYER		0xf10a  // MEDIA_TYPE_LONG
#define MEDIA_MP3ID3_NUMFRAMES		0xf10b  // MEDIA_TYPE_LONG
#define MEDIA_MP3ID3_BITRATE		   0xf10c  // MEDIA_TYPE_LONG
#define MEDIA_MP3ID3_SAMPLERATE		0xf10d  // MEDIA_TYPE_LONG
#define MEDIA_MP3ID3_CHANNELMODE	   0xf10e  // MEDIA_TYPE_LONG
#define MEDIA_MP3ID3_EMPHASIS		   0xf10f  // MEDIA_TYPE_LONG
#define MEDIA_MP3ID3_COPYRIGHTED	   0xf110  // MEDIA_TYPE_LONG
#define MEDIA_MP3ID3_ORIGINAL		   0xf111  // MEDIA_TYPE_LONG
