//-------------------------------------------------------------------
// Metadata.h
//-------------------------------------------------------------------
// ACD SYSTEMS INTERNATIONAL INC.
// Copyright © 1994 - 2020 ACD Systems International Inc.
// All Rights Reserved
//
// NOTICE: ACD Systems International Inc. ("ACD") permits you to use,
// modify, and distribute this file in accordance with the terms of
// the ACDSee SDK license agreement accompanying it. If you have 
// received this file from a source other than ACD, then your use,
// modification, or distribution of it requires the prior written 
// permission of ACD.
//-------------------------------------------------------------------

/**********************************************************************************

   Header containing GUID's and classes for handling image metadata.

**********************************************************************************/

#pragma once

#include <windows.h>
#include <initguid.h>

// METADATA_COMMENT
// 1FA6DDAE-02FF-4E86-9DAA-64B7E4253339
DEFINE_GUID(METADATA_COMMENT, 0x1FA6DDAE, 0x02FF, 0x4E86, 0x9D, 0xAA, 0x64, 0xB7, 0xE4, 0x25, 0x33, 0x39);

// METADATA_EXIF
// 51493775-A5B1-4CA7-BB31-7C46856D35B9
DEFINE_GUID(METADATA_EXIF, 0x51493775, 0xA5B1, 0x4CA7, 0xBB, 0x31, 0x7C, 0x46, 0x85, 0x6D, 0x35, 0xB9);

// METADATA_IPTC
// 0FE63733-3850-4A82-8259-3D7176D85DEA
DEFINE_GUID(METADATA_IPTC, 0x0FE63733, 0x3850, 0x4A82, 0x82, 0x59, 0x3D, 0x71, 0x76, 0xD8, 0x5D, 0xEA);

// METADATA_COLORPROFILE
// 5374B17D-FDBE-4355-82CB-BA0D9A85EC4F
DEFINE_GUID(METADATA_COLORPROFILE, 0x5374B17D, 0xFDBE, 0x4355, 0x82, 0xCB, 0xBA, 0x0D, 0x9A, 0x85, 0xEC, 0x4F);

// METADATA_AUDIO
// 71E96A15-9A0D-400A-90DA-32351FC20C82
DEFINE_GUID(METADATA_AUDIO, 0x71E96A15, 0x9A0D, 0x400A, 0x90, 0xDA, 0x32, 0x35, 0x1F, 0xC2, 0x0C, 0x82);

// METADATA_MULTIMEDIA
// 304286F5-B866-43C8-A35C-4C1309C8E0FD
DEFINE_GUID(METADATA_MULTIMEDIA, 0x304286F5, 0xB866, 0x43C8, 0xA3, 0x5C, 0x4C, 0x13, 0x09, 0xC8, 0xE0, 0xFD);

// METADATA_XMP
// 3B66F00F-4AD0-11D9-B08F-00E018763F3C
DEFINE_GUID(METADATA_XMP, 0x3B66F00F, 0x4AD0, 0x11D9, 0xB0, 0x8F, 0x00, 0xE0, 0x18, 0x76, 0x3F, 0x3C);

// METADATA_IMAGERESOLUTION
// 5BE48C5C-C927-4C9E-B67B-3FBAC19CA460
DEFINE_GUID(METADATA_IMAGERESOLUTION, 0x5BE48C5C, 0xC927, 0x4C9E, 0xB6, 0x7B, 0x3F, 0xBA, 0xC1, 0x9C, 0xA4, 0x60);

// METADATA_ACDDEFINEDXML
DEFINE_GUID(METADATA_ACDDEFINEDXML, 0xcc3bf24, 0x281, 0x11db, 0x96, 0x2a, 0x0, 0xe0, 0x81, 0x61, 0x16, 0x5f);

// METADATA_ACD
// {A9FA7EA2-C613-42f3-8845-D15B96454015}
DEFINE_GUID(METADATA_ACD, 0xa9fa7ea2, 0xc613, 0x42f3, 0x88, 0x45, 0xd1, 0x5b, 0x96, 0x45, 0x40, 0x15);

// METADATA_IPTC_DIGEST
// {166112C6-DDC4-4260-98D4-654AC4593B0D}
DEFINE_GUID(METADATA_IPTCDIGEST, 0x166112c6, 0xddc4, 0x4260, 0x98, 0xd4, 0x65, 0x4a, 0xc4, 0x59, 0x3b, 0xd);

// METADATA_SNAPSHOTS
// {C8C66AF9-D8AC-4DAC-82B5-0575D7D4EB38}
DEFINE_GUID(METADATA_SNAPSHOTS, 0xc8c66af9, 0xd8ac, 0x4dac, 0x82, 0xb5, 0x5, 0x75, 0xd7, 0xd4, 0xeb, 0x38);

#include "Exif.h"
#include "Iptc.h"
#include "Embedded.h"
#include "Media.h"
#include "Acd.h"

/////////////////////
// MD_MetadataItem //
/////////////////////
// An Item of Metadata
class MD_MetadataItem
{
public:
   virtual GUID  GetMetadataType() = 0;
   virtual void* GetMetadataData() = 0;
   virtual void  Release() = 0;
   virtual int   AddRef() = 0;
protected:
   virtual ~MD_MetadataItem() {};
};

/////////////////////
// MD_MetadataList //
/////////////////////
// List of metadata in an image
class MD_MetadataList
{
public:
   virtual int GetMetadataCount() = 0;
   virtual MD_MetadataItem* GetMetadataItem(int idx) = 0;
   virtual int AddMetadataItem(MD_MetadataItem* pItem) = 0;
   virtual void RemoveItem(int idx) = 0;
   virtual void RemoveItem(GUID guidType) = 0;
   virtual void Release() = 0;
   virtual int AddRef() = 0;
protected:
   virtual ~MD_MetadataList() {};
};
